==> herrie-0.1.tar.gz: herrie-0.1/src/audio/vorbis/vorbisfile.c <==
/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: vorbisfile.c 479 2006-08-01 07:06:42Z ed $
 */
/**
 * @file audio/vorbis/vorbisfile.c
 */

#include <stdlib.h>
#include <string.h>

#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>

#include <audio/vorbis/driver.h>
#include <audio/output/output.h>

#include <missing/missing.h>

/**
 * @brief Read tags from Ogg Vorbis file and store them in the audio
 *        file handle.
 */
static void
vorbis_read_comments(struct audio_file *fd)
{
	OggVorbis_File *vfp = fd->drv_data;
	struct vorbis_comment *cmt;
	int i;
	char *tag, *value;

	if ((cmt = ov_comment(vfp, -1)) == NULL)
		return;
	
	for (i = 0; i < cmt->comments; i++) {
		value = cmt->user_comments[i];
		tag = strsep(&value, "=");
		if (value == NULL)
			continue;

		if (strcmp(tag, "artist") == 0)
			fd->tag.artist = value;
		else if (strcmp(tag, "title") == 0)
			fd->tag.title = value;
		else if (strcmp(tag, "album") == 0)
			fd->tag.album = value;
	}
}

int
vorbis_open(struct audio_file *fd)
{
	OggVorbis_File *vfp;
	vorbis_info *info;

	if ((vfp = malloc(sizeof(OggVorbis_File))) == NULL)
		return (-1);

	if (ov_open(fd->fp, vfp, NULL, 0) != 0) {
		free(vfp);
		return (-1);
	}

	info = ov_info(vfp, -1);

	fd->drv_data = vfp;
	fd->srate = info->rate;
	fd->time_len = ov_time_total(vfp, -1);

	vorbis_read_comments(fd);
	
	return (0);
}

void
vorbis_close(struct audio_file *fd)
{
	OggVorbis_File *vfp = fd->drv_data;

	ov_clear(vfp);
	free(vfp);

	/* libvorbisfile already closes the file for us */
	fd->fp = NULL;
}

size_t
vorbis_read(struct audio_file *fd, void *buf)
{
	OggVorbis_File *vfp = fd->drv_data;
	size_t ret;

	/* Return 16 bits signed little endian */
	ret = ov_read(vfp, buf, AUDIO_OUTPUT_BUFLEN, 0, 2, 1, NULL);
	fd->time_cur = ov_time_tell(vfp);

	return (ret);
}

void
vorbis_seek(struct audio_file *fd, double len)
{
	OggVorbis_File *vfp = fd->drv_data;

	len += fd->time_cur;

	/* Negative length */
	if (len < 0)
		len = 0;
	else if (len > fd->time_len)
		len = fd->time_len;

	ov_time_seek(vfp, len);
}

==> herrie-0.6.tar.gz: herrie-0.6/src/audio/vorbis/vorbisfile.c <==
/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */
/**
 * @file audio/vorbis/vorbisfile.c
 */

#include <glib.h>

#include <string.h>

#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>

#include <audio/vorbis/driver.h>
#include <audio/output/output.h>

/**
 * @brief Read tags from Ogg Vorbis file and store them in the audio
 *        file handle.
 */
static void
vorbis_read_comments(struct audio_file *fd)
{
	OggVorbis_File *vfp = fd->drv_data;
	struct vorbis_comment *cmt;
	int i;
	char *tag, *value;

	if ((cmt = ov_comment(vfp, -1)) == NULL)
		return;
	
	for (i = 0; i < cmt->comments; i++) {
		tag = cmt->user_comments[i];
		value = strchr(tag, '=');
		if (value == NULL) {
			/* No foo=bar format */
			continue;
		} else {
			/* Split at the '=' character */
			*value++ = '\0';
		}

		if (strcmp(tag, "artist") == 0)
			fd->tag.artist = value;
		else if (strcmp(tag, "title") == 0)
			fd->tag.title = value;
		else if (strcmp(tag, "album") == 0)
			fd->tag.album = value;
	}
}

int
vorbis_open(struct audio_file *fd)
{
	OggVorbis_File *vfp;
	vorbis_info *info;

	vfp = g_malloc(sizeof(OggVorbis_File));
	if (ov_open(fd->fp, vfp, NULL, 0) != 0) {
		g_free(vfp);
		return (-1);
	}

	info = ov_info(vfp, -1);

	fd->drv_data = vfp;
	fd->srate = info->rate;
	fd->channels = 2; /* XXX */
	fd->time_len = ov_time_total(vfp, -1);

	vorbis_read_comments(fd);
	
	return (0);
}

void
vorbis_close(struct audio_file *fd)
{
	OggVorbis_File *vfp = fd->drv_data;

	ov_clear(vfp);
	g_free(vfp);

	/* libvorbisfile already closes the file for us */
	fd->fp = NULL;
}

size_t
vorbis_read(struct audio_file *fd, void *buf)
{
	OggVorbis_File *vfp = fd->drv_data;
	size_t ret;

	/* Return 16 bits signed little endian */
	ret = ov_read(vfp, buf, AUDIO_OUTPUT_BUFLEN, 0, 2, 1, NULL);
	fd->time_cur = ov_time_tell(vfp);

	return (ret);
}

void
vorbis_seek(struct audio_file *fd, double len)
{
	OggVorbis_File *vfp = fd->drv_data;

	len += fd->time_cur;

	/* Negative length */
	if (len < 0)
		len = 0;
	else if (len > fd->time_len)
		len = fd->time_len;

	ov_time_seek(vfp, len);
}
